
#############################################################
#                                                           #
# bandcamp-get - automated downloading of bandcamp music    #
# written by Hunter Hammond (huntrar@gmail.com)             #
#                                                           #
#############################################################


import argparse
from random import randint
import requests
import sys
import time
from . import __version__

import lxml.html
from selenium import webdriver
from selenium.webdriver.common.keys import Keys


def get_parser():
    parser = argparse.ArgumentParser(description='automated downloading of bandcamp music')
    parser.add_argument('user', metavar='USER', type=str, nargs='?',
                         help='bandcamp user to download from')
    parser.add_argument('-b', '--browser', type=str,
                         help='enter chrome or firefox, defaults to firefox')
    parser.add_argument('-e', '--email', type=str,
                         help='use your own email instead of a throwaway')
    parser.add_argument('-v', '--version', help='display current version',
                        action='store_true')
    return parser


def command_line_runner():
    parser = get_parser()
    args = vars(parser.parse_args())

    if args['version']:
        print(__version__)
        return

    if not args['user']:
        parser.print_help()
        return

    args['zip_code'] = randint(10000, 99999)
    args['album_links'] = get_album_links(args)
    if args['email']:
        download(args)
    else:
        auto_download(args)


def get_driver(browser):
    if browser and 'chrome' in browser.lower():
        return webdriver.Chrome()
    else:
        return webdriver.Firefox()


def get_album_links(args):
    seed_url = 'https://%s.bandcamp.com' % args['user']
    r = requests.get(seed_url, headers={'User-agent':'Firefox'})
    l = lxml.html.fromstring(r.text)
    all_links = l.xpath('//li/a[@href]/@href')
    album_links = []

    for link in all_links:
        if '/album/' in link:
            album_links.append(seed_url + link)
    return album_links


def auto_download(args):
    e_driver = get_driver(args['browser']) # Email driver
    e_driver.implicitly_wait(1)
    e_driver.get('https://www.guerrillamail.com/')
    args['email'] = e_driver.find_element_by_xpath('//span[@id="email-widget"]').text
    mail_cache = set()
    download(args)
    check_email(e_driver, mail_cache)
    try:
        e_driver.close()
    except Exception:
        pass
        

def check_email(e_driver, mail_cache):
    emails = e_driver.find_elements_by_xpath('//tbody[@id="email_list"]/tr')
    for mail in emails:
        download_mail_link(e_driver, mail_cache, mail)

    # run again in case any emails came in late, ignoring already read emails
    e_driver.get('https://www.guerrillamail.com/')
    emails = e_driver.find_elements_by_xpath('//tbody[@id="email_list"]/tr')
    for mail in emails:
        download_mail_link(e_driver, mail_cache, mail)

    e_driver.quit()


def download_mail_link(e_driver, mail_cache, mail):
    mail_window = e_driver.window_handles[0]
    try:
        mail.click()
        try:
            dl_element = e_driver.find_element_by_xpath('//div[@class="email_body"]/div//a[@target="_blank"]')
            if dl_element.text not in mail_cache:
                mail_cache.add(dl_element.text)
                dl_element.click()
                dl_window = e_driver.window_handles[1]
                e_driver.switch_to_window(dl_window)
                e_driver.find_element_by_xpath('//a[@class="downloadGo"]').click()
                e_driver.close()
                e_driver.switch_to_window(mail_window)
                e_driver.find_element_by_xpath('//a[@id="back_to_inbox_link"]').click()
        except Exception:
            e_driver.switch_to_window(mail_window)
            e_driver.find_element_by_xpath('//a[@id="back_to_inbox_link"]').click()
    except Exception:
        pass


def download(args):
    driver = get_driver(args['browser'])
    driver.implicitly_wait(1)
    for link in args['album_links']:
        if not download_link(args, driver, link):
            break
    try:
        driver.close()
    except Exception:
        pass


def download_link(args, driver, link):
        try:
            driver.get(link)
            driver.find_element_by_xpath("//button[@class='download-link buy-link']").click()
        except Exception:
            return False

        print('Downloading %s' % link)
        try:
            element = driver.find_element_by_xpath('//input[@id="userPrice"]')
            element.send_keys('0')
        except Exception:
            pass

        try:
            element = driver.find_element_by_xpath('//input[@id="fan_email_address"]')
            element.send_keys(args['email'])

            element = driver.find_element_by_id('fan_email_postalcode')
            element.send_keys(args['zip_code'])
        except Exception:
            pass

        try:
            element.send_keys(Keys.RETURN)
        except Exception:
            pass

        try:
            driver.find_element_by_xpath('//a[@class="downloadGo"]').click()
            time.sleep(1)
        except Exception:
            pass

        return True


if __name__ == '__main__':
    command_line_runner()
